@@ -2,7 +2,7 @@
 
 # py_tools_ds
 #
-# Copyright (C) 2019  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
+# Copyright (C) 2016-2021  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
 #
 # This software was developed within the context of the GeoMultiSens project funded
 # by the German Federal Ministry of Education and Research
@@ -32,7 +32,7 @@ from osgeo import gdal, gdalnumeric
 
 from ...dtypes.conversion import dTypeDic_NumPy2GDAL
 from ..projection import WKT2EPSG, isProjectedOrGeographic, prj_equal
-from ..coord_trafo import pixelToLatLon
+from ..coord_trafo import pixelToLatLon, transform_any_prj
 from ..coord_calc import corner_coord_to_minmax, get_corner_coordinates
 from ...io.raster.gdal import get_GDAL_ds_inmem
 from ...processing.progress_mon import ProgressBar
@@ -67,7 +67,6 @@ def warp_ndarray_rasterio(ndarray, in_gt, in_prj, out_prj, out_gt=None, outRowsC
     :return out_gt:             warped gdal GeoTransform
     :return out_prj:            warped projection as WKT string
     """
-
     if not find_loader('rasterio'):
         raise ImportError('This function requires rasterio. You need to install it manually '
                           '(conda install -c conda-forge rasterio). It is not automatically installed.')
@@ -75,7 +74,7 @@ def warp_ndarray_rasterio(ndarray, in_gt, in_prj, out_prj, out_gt=None, outRowsC
     # NOTE: rasterio seems to increase the number of objects with static TLS
     #       There is a maximum number and if this is exceeded an ImportError is raised:
     #       ImportError: dlopen: cannot load any more object with static TLS
-    #       - see also: https://gitext.gfz-potsdam.de/danschef/py_tools_ds/issues/8
+    #       - see also: https://git.gfz-potsdam.de/danschef/py_tools_ds/issues/8
     #       - NOTE: importing rasterio AFTER pyresample (which uses pykdtree) seems to solve that too
     #       => keep the rasterio import within the function locals to avoid not needed imports
     import rasterio
@@ -254,7 +253,7 @@ def warp_ndarray(ndarray, in_gt, in_prj=None, out_prj=None, out_dtype=None,
                  out_alpha=False, targetAlignedPixels=False, gcpList=None, polynomialOrder=None, options=None,
                  transformerOptions=None, warpOptions=None, CPUs=1, warpMemoryLimit=0, progress=True, q=False):
     # type: () -> (np.ndarray, tuple, str)
-    """
+    r"""
 
     :param ndarray:             numpy array to be warped (or a list of numpy arrays (requires lists for in_gt/in_prj))
     :param in_gt:               input GDAL geotransform (or a list of GDAL geotransforms)
@@ -430,7 +429,9 @@ def warp_ndarray(ndarray, in_gt, in_prj=None, out_prj=None, out_dtype=None,
     #    in_ds.SetMetadata(rpc, "RPC")
     #    transformerOptions = ['RPC_DEM=data/warp_52_dem.tif']
 
-    if CPUs is None or CPUs > 1:
+    # use GDALs multiprocessing as long as the current process is not a child process of a multiprocessing main process
+    # - otherwise, gdal.Warp() hangs with GDAL versions from 3.2.1 and above (does not allow sub-multiprocessing)
+    if (CPUs is None or CPUs > 1) and multiprocessing.current_process().name == 'MainProcess':
         gdal.SetConfigOption('GDAL_NUM_THREADS', str(CPUs if CPUs else multiprocessing.cpu_count()))
 
         # gdal.SetConfigOption('GDAL_CACHEMAX', str(800))
@@ -502,9 +503,17 @@ def warp_ndarray(ndarray, in_gt, in_prj=None, out_prj=None, out_dtype=None,
 
     # output bounds verification
     if out_bounds:
+        if out_bounds_prj and not prj_equal(out_bounds_prj, res_prj):
+            out_xmin, out_ymin = transform_any_prj(out_bounds_prj, res_prj, *out_bounds[:2])
+            out_xmax, out_ymax = transform_any_prj(out_bounds_prj, res_prj, *out_bounds[2:])
+
+        else:
+            out_xmin, out_ymin, out_xmax, out_ymax = out_bounds
+
         xmin, xmax, ymin, ymax = \
             corner_coord_to_minmax(get_corner_coordinates(gt=res_gt, rows=res_arr.shape[0], cols=res_arr.shape[1]))
-        if False in np.isclose(out_bounds, (xmin, ymin, xmax, ymax)):
-            warnings.warn('The output bounds of warp_ndarray do not match the requested bounds!')
+
+        if False in np.isclose((out_xmin, out_ymin, out_xmax, out_ymax), (xmin, ymin, xmax, ymax)):
+            warnings.warn('The output bounds of warp_ndarray do not exactly match the requested bounds!')
 
     return res_arr, res_gt, res_prj
